//----------------------------------------------------------------------------

//Sine wave generator SG-642
//  

//----------------------------------------------------------------------------

#include "Main.h"
#include "Control.h"
#include "Menu.h"

//----------------------------- : -----------------------------------

#define BAUD1 1250000 //   , 
#define CO_TO     100 //   , 

#define C_SET_F	0x01 //SET FREQUENCY command
#define C_SET_P	0x02 //SET PHASE command
#define C_SET_D	0x03 //SET DELTA PHASE command
#define C_SET_S	0x04 //SET SHAPE command

#define CHAN_A	0x10 //channel A mask
#define CHAN_B	0x20 //channel B mask

#define UBRRV1 (int)((F_CLK * 1000000.0)/(16.0 * BAUD1) - 0.5) //
#define TX_CMD (1 << RXEN1) | (1 << TXEN1) | (1 << UCSZ12) | (1 << TXB81)
#define TX_DAT (1 << RXEN1) | (1 << TXEN1) | (1 << UCSZ12)

#define DDS_DIV 97              //    DDS   
#define DDS_RES 17179869184LL   // DDS (2^34)
#define CALF_RES 10000000LL     //   (0.1 PPM)
#define PHS_RES (DDS_RES / 256) //   DDS (2^26)

#define KF ((long long)(DDS_DIV * DDS_RES / 2048))
#define DF ((long long)(F_CLK * 1000000000LL / 2048)) 

#define AMP_MAX 1000 //  , 0.01 
#define DAC_RES 4095 //  
#define DAC_C 0xD000 //  
#define DAC_A 0x1000 //  A  
#define DAC_B 0x4000 //  B  

#define SIGCAL  0xBABE // EEPROM  
#define CALL_RES 10000 //   (0.01%)

//----------------------------- : ----------------------------------

static char Shape[OUTS];     // 
static int  Phase[OUTS];     //  , MIN_P..MAX_P x0.1
static char Atten[OUTS];     //  , ATT_OFF..ATT_0DB

static int  Cal_F;           // , 999 (99.9 ppm)
static int  Cal_Sin[OUTS];   //  sin, 999 (9.99%)
static int  Cal_Sqr[OUTS];   //  sqr, 999 (9.99%)

__no_init __eeprom int  ESigCal;   // EEPROM  
__no_init __eeprom int  ECal_F;    //   EEPROM
__no_init __eeprom int  ECal_Sin[OUTS]; //  sin  EEPROM
__no_init __eeprom int  ECal_Sqr[OUTS]; //  sqr  EEPROM

//--------------------------  : ------------------------------

void LoadDAC(unsigned int d); //    SPI
void CoByte(char b);          //  

//-----------------------  : --------------------------

void Control_Init(void)
{
  // UART1,  9 :
	UCSR1C = (1 << UCSZ11) | (1 << UCSZ10);
  // 9  :
	UCSR1B = TX_DAT;
  //  :
  UBRR1L = LO(UBRRV1);                 
  UBRR1H = HI(UBRRV1);
  //SPI enable, MSB first, master, falling edge:
  SPCR = (1 << SPE0) | (1 << CPOL0) | (1 << MSTR0);
  //double SPI speed:
  SPSR = (1 << SPI2X0);
  // :
  Control_ReadCal();
  // :
  for(char i = 0; i < OUTS; i++)
  {
    Shape[i] = SH_S;
    Phase[i] = 0;
    Atten[i] = ATT_OFF;
  }
  //DAC SDO2 = 1, SDO1 = 1, DSY = 0, HCLR = 0, SCLK = 0:
  LoadDAC(0x0C00 | DAC_C);
  //co-processor power-up delay 100 mS:
  Delay_ms(100);
}

//---------------------    EEPROM: -------------------------

void Control_ReadCal(void)
{
  if(ESigCal != SIGCAL) //  ,
  {
    // :
    Cal_F = NOM_CALIB;
    for(char i = 0; i < OUTS; i++)
    {
      Cal_Sin[i] = NOM_CALIB;
      Cal_Sqr[i] = NOM_CALIB;
    }
    Control_SaveCal(); //   EEPROM
  }
  else //  ,
  {
    Cal_F = ECal_F; //  
    for(char i = 0; i < OUTS; i++) //  
    {
      Cal_Sin[i] = ECal_Sin[i];
      Cal_Sqr[i] = ECal_Sqr[i];
    }
  }
}

//-------------------    EEPROM: ------------------------

void Control_SaveCal(void)
{
  if(ECal_F != Cal_F) ECal_F = Cal_F; //  
  for(char i = 0; i < OUTS; i++)      //  
  {
    if(ECal_Sin[i] != Cal_Sin[i]) ECal_Sin[i] = Cal_Sin[i];
    if(ECal_Sqr[i] != Cal_Sqr[i]) ECal_Sqr[i] = Cal_Sqr[i];
    __watchdog_reset();
  }
  if(ESigCal != SIGCAL) ESigCal = SIGCAL; // 
}

//------------------   : ---------------------

int Control_GetK(char s)
{
  long k = 0;
  switch(s)
  {
  case CAL_F: k = Cal_F;
              break;
  case CAL_A: if(Shape[OUT_A] == SH_M) k = Cal_Sqr[OUT_A];
                else k = Cal_Sin[OUT_A];
              break;
  case CAL_B: if(Shape[OUT_B] == SH_M) k = Cal_Sqr[OUT_B];
                else k = Cal_Sin[OUT_B];
              break;
  }
  return(k);
}

//-----------------   : -------------------

void Control_SetK(int k, char s)
{
  switch(s)
  {
  case CAL_F: Cal_F = k;
              break;
  case CAL_A: if(Shape[OUT_A] == SH_M) Cal_Sqr[OUT_A] = k;
                else Cal_Sin[OUT_A] = k;
              break;
  case CAL_B: if(Shape[OUT_B] == SH_M) Cal_Sqr[OUT_B] = k;
                else Cal_Sin[OUT_B] = k;
              break;
  }
}

//-----------------------   : ---------------------

void CoCommand(char cmd, long d)
{
	UCSR1B = TX_CMD;  //  (TXB81 = 1)
  CoByte(cmd);
	UCSR1B = TX_DAT;  //  (TXB81 = 0)
  CoByte(BYTE1(d));
  CoByte(BYTE2(d));
  CoByte(BYTE3(d));
  CoByte(BYTE4(d));
}

//---------------------   : -------------------------

void CoByte(char b)
{
  char CoInt = Pin_CINT; //  Pin_CINT
  char TmOut = (char)(CO_TO * F_CLK / 9); //  (while - 9 )
  UDR1 = b;              //   
  while(Pin_CINT == CoInt && TmOut--); //  Pin_CINT
}

//------------------     SPI: ------------------------

void LoadDAC(unsigned int d)
{
  Port_SLOAD_0;
  SPDR = HI(d);
  while(!(SPSR & (1 << SPIF0)));
  SPDR = LO(d);
  while(!(SPSR & (1 << SPIF0)));
  Port_SLOAD_1;
}

//----------------------------  : ------------------------------

//ch - : OUT_A, OUT_B
//s - : SH_S, SH_M

void Control_SetS(char ch, char s)
{
  if(ch < OUTS)
  {
    if(Shape[ch] != s) // 
    Shape[ch] = s;     // 
    if(ch == OUT_A) LoadDAC(0 | DAC_A); //   A
    if(ch == OUT_B) LoadDAC(0 | DAC_B); //   B
  }
  char cmd;
  if(ch == OUT_A) cmd = C_SET_S + CHAN_A;
  if(ch == OUT_B) cmd = C_SET_S + CHAN_B;
  CoCommand(cmd, s); //   
  Delay_ms(15);      //   
}

//---------------------------  : -----------------------------

//ch - : OUT_A, OUT_B, OUTS
//f - : 0..MAX_F

void Control_SetF(char ch, long f)
{
  char cmd; long code;
  if(ch == OUT_A) cmd = C_SET_F + CHAN_A;
  if(ch == OUT_B) cmd = C_SET_F + CHAN_B;
  if(ch == OUTS)  cmd = C_SET_F + CHAN_A + CHAN_B;
  //     :
  code = (long long)f * (KF + Cal_F * KF / CALF_RES) / DF;
  CoCommand(cmd, code); //   
}

//-------------------------   : ---------------------------

//ch - : OUT_A, OUT_B, OUTS
//p -  : MAX_P

void Control_SetP(char ch, int p)
{
  char cmd; int dp = p; long code;
  //     dp   A  B:
  if(ch == OUT_A) { cmd = C_SET_D + CHAN_A; dp = p - Phase[ch]; Phase[ch] = p; }
  if(ch == OUT_B) { cmd = C_SET_D + CHAN_B; dp = p - Phase[ch]; Phase[ch] = p; }
  //    ,   dp = p:
  if(ch == OUTS)  { cmd = C_SET_P + CHAN_A + CHAN_B;
                    Phase[OUT_A] = p; Phase[OUT_B] = p; } 
  code = (long long)dp * PHS_RES / MAX_P;
  CoCommand(cmd, code); //   
}

//----------------------------  : -----------------------------

//ch - : OUT_A, OUT_B
//a - : 0..MAX_DAC

void Control_SetA(char ch, long a)
{
  if(ch >= OUTS) return; //  
  //        :
  long Cal = (Shape[ch] == SH_M)? Cal_Sqr[ch] : Cal_Sin[ch];
  //  DAC   :
  unsigned int code = (long)a * (DAC_RES + Cal * DAC_RES / CALL_RES) / MAX_DAC;
  if(code > DAC_RES) code = DAC_RES;
  //  ,  :
  if(Atten[ch] == ATT_OFF) code = 0;
  if(ch == OUT_A) LoadDAC(code | DAC_A); // DAC  A
  if(ch == OUT_B) LoadDAC(code | DAC_B); // DAC  B
}

//--------------------------  : --------------------------

//ch - : OUT_A, OUT_B
//r - : ATT_OFF, ATT_40DB, ATT_20DB, ATT_0DB

void Control_SetR(char ch, char r)
{
  if(ch >= OUTS) return;     //  
  if(r == Atten[ch]) return; // 
  Atten[ch] = r;             //  
  if(ch == OUT_A)
    switch(r)
    {
    //   A:  
    case ATT_OFF:  Port_RELAY1_0; Port_RELAY2_0; Port_RELAY3_0; break;
    case ATT_40DB: Port_RELAY1_0; Port_RELAY2_0; Port_RELAY3_1; break;
    case ATT_20DB: Port_RELAY1_0; Port_RELAY2_1; Port_RELAY3_0; break;
    case ATT_0DB:  Port_RELAY1_1; Port_RELAY2_0; Port_RELAY3_0; break;
    }
  if(ch == OUT_B)
    switch(r)
    {
    //   B:  
    case ATT_OFF:  Port_RELAY4_0; Port_RELAY5_0; Port_RELAY6_0; break;
    case ATT_40DB: Port_RELAY4_0; Port_RELAY5_0; Port_RELAY6_1; break;
    case ATT_20DB: Port_RELAY4_0; Port_RELAY5_1; Port_RELAY6_0; break;
    case ATT_0DB:  Port_RELAY4_1; Port_RELAY5_0; Port_RELAY6_0; break;
    }
}

//----------------------------------------------------------------------------
