//----------------------------------------------------------------------------

//Project:         Sine wave generator SG-642
//Compiler:        IAR EWAVR 4.21A
//Microcontroller: ATmega324P
//E-mail:          wubblick@yahoo.com

//Fuses: LO: 0x00, HI: 0x00, EXT: 0x00

//----------------------------------------------------------------------------

#include "Main.h"
#include "Port.h"
#include "Commands.h"
#include "Encoder.h"
#include "Keyboard.h"
#include "Lcd133x64.h"
#include "Sound.h"
#include "Menu.h"
#include "Control.h"

//------------------------------- : --------------------------------

volatile bool fTick;               //  watchdog-
bool tick;                         //    

//----------------------------------------------------------------------------

void main(void);                   // 
void Main_Wdt_Init(void);          // watchdog-
void Main_Rst_Wdt(bool t);         // watchdog-
void Main_Ports_Init(void);        // 
void Main_Timer_Init(void);        //  
#pragma vector = TIMER1_COMPA_vect
__interrupt void Sys_Timer(void);  //  
__monitor bool Main_GetTick(void); //  

//----------------------------------------------------------------------------
//-------------------------  : ------------------------------
//----------------------------------------------------------------------------

void main(void)
{
// :
  Main_Wdt_Init();          // watchdog-
  Main_Ports_Init();        // 
  Main_Timer_Init();        //  
  Port_Init();              // 
  Encoder_Init();           // 
  Keyboard_Init();          // 
  LCD_Init();               // LCD
  Sound_Init();             //  
  Control_Init();           // 
  Menu_Init();              // 
  __enable_interrupt();     // 
// :
  while(1)
  {
    tick = Main_GetTick();  //  
    Commands_Exe();         //  
    Encoder_Exe();          // 
    Keyboard_Exe(tick);     // 
    Menu_Exe(tick);         // 
    Main_Rst_Wdt(tick);     // watchdog-
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//---------------------  watchdog-: ----------------------

void Main_Wdt_Init(void)
{
  __watchdog_reset();
  WDTCSR = (1 << WDCE) | (1 << WDE);
  WDTCSR = (1 << WDE) | (1 << WDP2); // watchdog (time-out 260 ms)
}

//------------------------  watchdog-: -------------------------

void Main_Rst_Wdt(bool t)
{
  if(t)                 //   , 
    __watchdog_reset(); // watchdog-
}

//-------------------------  : ----------------------------

void Main_Ports_Init(void)
{
  DDRA  = I_DDRA;
  PORTA = I_PORTA;
  DDRB  = I_DDRB;
  PORTB = I_PORTB;
  DDRC  = I_DDRC;
  PORTC = I_PORTC;
  DDRD  = I_DDRD;
  PORTD = I_PORTD;
}

//------------------    ------------------------

void Main_Timer_Init(void)
{
  TCCR1A = 0;
  TCCR1B = (1 << WGM12) | (1 << CS10); //  3: CTC, CK/1
  OCR1A = (int)(F_CLK * T_SYS) - 1; //  
  TIFR1 = (1 << OCF1A);    //  
  TIMSK1 |= (1 << OCIE1A); //   
  fTick = 1;               //   
}

//--------------------   : --------------------------

bool Main_GetTick(void)
{
  if(!fTick) return(0);
  fTick = 0;
  return(1);
}

//-------------------------   3: ----------------------------

#pragma vector = TIMER1_COMPA_vect
__interrupt void Sys_Timer(void)
{
  fTick = 1;   //    
}

//-----------------   : ----------------------

void Delay_ms(int d)
{
  while(d)
  {
    Delay_us(1000);     // 1 
    __watchdog_reset(); // watchdog-
    d--;
  }
}

//----------------------------------------------------------------------------
