//----------------------------------------------------------------------------

//Sine wave generator SG-642
// : header file

//----------------------------------------------------------------------------

#ifndef MainH
#define MainH

//----------------------------------------------------------------------------

#include <iom324p.h>
#include <intrinsics.h>
#include <stdbool.h>
#include <stdlib.h>

//----------------------------------------------------------------------------

#define VER "1.3"     //  firmware

#define	F_CLK 20.0000 //clock frequency, MHz
#define T_SYS 1000    //system timer, uS

//----------------------------------------------------------------------------

// :

//I - 
//O - 
//B -  

//L -   
//H -   
//X -    
//A -  

//-------------------------------  A: ------------------------------------

#define KEY_E    (1 << PA0) //IL -  
#define ENC_F1   (1 << PA1) //IL -   F1
#define ENC_F2   (1 << PA2) //IL -   F2
#define RES      (1 << PA3) //OL - LCD reset
#define NC_PA4   (1 << PA4) //IL -  
#define NC_PA5   (1 << PA5) //IL -  
#define DET_A    (1 << PA6) //IA -   A (ADC6)
#define DET_B    (1 << PA7) //IA -   B (ADC7)

//:
#define I_DDRA  (RES)
//  (in)    (out):
#define I_PORTA (KEY_E | ENC_F1 | ENC_F2 | NC_PA4 | NC_PA5)
// :
#define MUX_DET_A ((1 << MUX2) | (1 << MUX1))               // ADC6
#define MUX_DET_B ((1 << MUX2) | (1 << MUX1) | (1 << MUX0)) // ADC7
//     :
#define Pin_KEY_E      (PINA & KEY_E)
#define Pin_ENC_F1     (PINA & ENC_F1)
#define Pin_ENC_F2     (PINA & ENC_F2)
#define Port_RES_0     (PORTA &= ~RES)
#define Port_RES_1     (PORTA |= RES)

//-------------------------------  B: ------------------------------------

#define KEY_C    (1 << PB0) //IL -  "CHAN"
#define KEY_U    (1 << PB1) //IL -  "UP"
#define KEY_D    (1 << PB2) //IL -  "DOWN"
#define KEY_S    (1 << PB3) //IL -  "STEP"
#define SLOAD    (1 << PB4) //OL -  
#define MOSI     (1 << PB5) //OH -  MOSI  SPI
#define MISO     (1 << PB6) //IH -  MISO  SPI
#define SCK      (1 << PB7) //OH -  SCK  SPI

//:
#define I_DDRB  (SLOAD | MOSI | SCK)
//  (in)    (out):
#define I_PORTB (KEY_C | KEY_U | KEY_D | KEY_S | SLOAD | MISO)
//     :
#define Pin_KEY_C      (PINB & KEY_C)
#define Pin_KEY_U      (PINB & KEY_U)
#define Pin_KEY_D      (PINB & KEY_D)
#define Pin_KEY_S      (PINB & KEY_S)
#define Port_SLOAD_0   (PORTB &= ~SLOAD)
#define Port_SLOAD_1   (PORTB |= SLOAD)

//-------------------------------  C: ------------------------------------

#define SCL      (1 << PC0) //OL -  SCL  I2C
#define SDA      (1 << PC1) //BL -  SDA  I2C
#define RELAY4   (1 << PC2) //OH -  4
#define RELAY5   (1 << PC3) //OH -  5
#define RELAY6   (1 << PC4) //OH -  6
#define RELAY3   (1 << PC5) //OH -  3
#define RELAY2   (1 << PC6) //OH -  2
#define RELAY1   (1 << PC7) //OH -  1

//:
#define I_DDRC  (SCL | RELAY4 | RELAY5 | RELAY6 | RELAY3 | RELAY2 | RELAY1)
//  (in)    (out):
#define I_PORTC (SCL | SDA)
//     :
#define Port_RELAY4_0  (PORTC &= ~RELAY4)
#define Port_RELAY4_1  (PORTC |= RELAY4)
#define Port_RELAY5_0  (PORTC &= ~RELAY5)
#define Port_RELAY5_1  (PORTC |= RELAY5)
#define Port_RELAY6_0  (PORTC &= ~RELAY6)
#define Port_RELAY6_1  (PORTC |= RELAY6)
#define Port_RELAY3_0  (PORTC &= ~RELAY3)
#define Port_RELAY3_1  (PORTC |= RELAY3)
#define Port_RELAY2_0  (PORTC &= ~RELAY2)
#define Port_RELAY2_1  (PORTC |= RELAY2)
#define Port_RELAY1_0  (PORTC &= ~RELAY1)
#define Port_RELAY1_1  (PORTC |= RELAY1)

//-------------------------------  D: ------------------------------------

#define PC_RXD0  (1 << PD0) //IL -  RXD0 (PC)
#define PC_TXD0  (1 << PD1) //OL -  TXD0 (PC)
#define PC_RXD1  (1 << PD2) //IL -  RXD1 (DDS)
#define PC_TXD1  (1 << PD3) //OL -  TXD1 (DDS)
#define CINT     (1 << PD4) //IL - DDS CINT
#define SND      (1 << PD5) //OL -  
#define NC_PD6   (1 << PD6) //IL -  
#define NC_PD7   (1 << PD7) //IL -  

//:
#define I_DDRD  (PC_TXD0 | PC_TXD1 | SND)
//  (in)    (out):
#define I_PORTD (PC_RXD0 | PC_TXD0 | PC_RXD1 | PC_TXD1 | CINT | SND | NC_PD6 | NC_PD7)
//     :
#define Pin_CINT       (PIND & CINT)
#define Port_SND       (PORTD & SND)
#define Port_SND_0     (PORTD &= ~SND)
#define Port_SND_1     (PORTD |= SND)

//--------------------------- : ------------------------------

#define HI(x) ((x >> 8) & 0xFF)
#define LO(x) (x & 0xFF)

#define BYTE1(x) (x & 0xFF)
#define BYTE2(x) ((unsigned int)(x >> 8) & 0xFF)
#define BYTE3(x) ((unsigned long)(x >> 16) & 0xFF)
#define BYTE4(x) ((unsigned long)(x >> 24) & 0xFF)

#define WORD(b1,b0) ((unsigned int)(b1 << 8) | b0)
#define DWORD(b3,b2,b1,b0) (((unsigned long)WORD(b3,b2) << 16) | WORD(b1,b0))

#define Delay_us(x) __delay_cycles((int)(x * F_CLK + 0.5))
#define ms2sys(x) ((int)(1E3 * x / T_SYS))

//--------------------------  : ------------------------------

void Delay_ms(int d);

//----------------------------------------------------------------------------

#endif
