//----------------------------------------------------------------------------

//Sine wave generator SG-642
//  

//----------------------------------------------------------------------------

#include "Main.h"
#include "Menu.h"
#include "Splash.h"
#include "Info.h"
#include "Keyboard.h"
#include "Encoder.h"
#include "Lcd133x64.h"
#include "Sound.h"
#include "Control.h"

//--------------------------- : -------------------------------------

#define SPL_TMV  1500  //  splash screen, 
#define INF_TMV  1500  //  splash screen, 
#define EXI_TMV  7500  //    , 

enum { MNU_NO,         //  
       MNU_SPLASH,     //  splash screen
       MNU_INFO,       //  
       MNU_OUTS,       //  OUTS
       MNU_STEPO,      //    OUTS
       MNU_SERV1,      // , page 1
       MNU_SERV2,      // , page 2
       MNU_CALIB,      // 
       MNU_SHOW };     //  splash  info

#define SIGPAR 0xBEDA  // EEPROM  params
#define SIGSET 0xBABA  // EEPROM  setup

enum { LIM_MIN,        //  
       LIM_NOM,        //  
       LIM_MAX,        //  
       LIMS };

//Min, Nom, Max    OUTS:

const __flash long LimValOuts[POUTS_MAX][LIMS] =
{  
  { MD_SPLIT, MD_SPLIT, MD_COMBO }, //Min, Nom, Max PAR_M
  {     SH_S,     SH_S,     SH_M }, //Min, Nom, Max PAR_S
  {    MIN_F,    NOM_F,    MAX_F }, //Min, Nom, Max PAR_F
  {    MIN_P,    NOM_P,    MAX_P }, //Min, Nom, Max PAR_P
  {    MIN_A,    NOM_A,    MAX_A }, //Min, Nom, Max PAR_A
  { REL_AUTO,  REL_0DB,  REL_0DB }, //Min, Nom, Max PAR_R
};

//Min, Nom, Max    OUTS:

const __flash long LimStpOuts[POUTS_MAX][LIMS] =
{
  {     1,    1,        1 }, //Min, Nom, Max STEP_M
  {     1,    1,        1 }, //Min, Nom, Max STEP_S
  {     0,    0, 10000000 }, //Min, Nom, Max STEP_F
  {     0,    0,     1000 }, //Min, Nom, Max STEP_P
  {     0,    0,      100 }, //Min, Nom, Max STEP_A
  {     1,    1,        1 }, //Min, Nom, Max STEP_R
};

#define SIGN_PLUS  0   //   
#define SIGN_MINUS 1   //   

#define PSS     10     // 
#define SAVED 0xFF     //  

//--------------------------- : ------------------------------------

static char KeyCode;    // 
static char DispMenu;   //  
static char Menu;       //  
static char SubMenu;    // 
static char RetMenu;    //  
static int  MenuTimer;  // 
static int  ExitTimer;  //  
static bool Lock;       //  
static bool OutEnable;  // 

static char SelPar;     // 
static long Buffer;     //   
static long Step;       //  
static long MinVal;     //  
static long MaxVal;     //  
static char Buff[10];   //   BCD-
static int  BuffK;      //   K

static char SelChan;    // 
static char Pars[OUTS]; //   
static long ValOuts[OUTS][POUTS_MAX]; //  OUTS
static long StpOuts[OUTS][POUTS_MAX]; //  OUTS
static bool Combo;         //  A+B
static char Units[OUTS];   //   (Amp/RMS)
static long DacCode[OUTS]; //  (0..MAX_DAC)
static char AttCode[OUTS]; //  (ATT_OFF..ATT_0DB)
static char AutoAtt[OUTS]; //    AUTO

static char SvPre;      //  
static char RdPre;      //  
static char SvReq;      //  

__no_init __eeprom int  ESigPar[PSS];     // EEPROM  params
__no_init __eeprom char EPars[PSS][OUTS]; //   
__no_init __eeprom long EValOuts[PSS][OUTS][POUTS_MAX]; //  OUTS
__no_init __eeprom long EStpOuts[PSS][OUTS][POUTS_MAX]; //  OUTS

static char Contr;      // LCD

__no_init __eeprom int  ESigSet;      // EEPROM  setup
__no_init __eeprom char EContr;       // LCD  EEPROM
__no_init __eeprom bool EMute;        //   EEPROM
__no_init __eeprom char EUnits[OUTS]; //    EEPROM

//--------------------------  : ------------------------------

void Mnu_Splash(bool ini);           //  
void Mnu_Info(bool ini);             //   
void Mnu_Outs(bool ini);             //  
void Mnu_StepO(bool ini);            //  
void Mnu_Serv1(bool ini);            // , page 1
void Mnu_Serv2(bool ini);            // , page 2
void Mnu_Calib(bool ini);            // 
void Mnu_Show(bool ini);             //  

void Show_Info(void);                //  Info
void Show_Val(char par);             //  
void Show_Freq(long f);              //  
void Show_FreqH(long f);             //   ( )
void Show_Phase(int p);              //  
void Show_Level(long v);             //  
void Show_Step(char par);            //  
void Show_FreqStep(long f);          //   
void Show_PhaseStep(int p);          //   
void Show_LevelStep(long v);         //   
void Show_Serv1(char s);             //  , page 1
void Show_Serv2(char s);             //  , page 2
void Show_Calib(char s);             //  
void Show_FCal(int v);               //  
void Show_ACal(int v);               //  

void Long2BCD(unsigned long x, char *buff); // Long2BCD
void Int2BCD(int x, char *buff);     // Int2BCD

void Setup_Read(void);               //   EEPROM
void Setup_Save(void);               //   EEPROM
bool Vals_Read(char n);              //   EEPROM
void Vals_Save(char n);              //   EEPROM

bool Edit_Val(char key);             // 
void ValUp(bool sign);               //  
void ValDn(bool sign);               //  

bool Edit_Step(char key);            // 
bool StepUp(char par, long *st);     // 
bool StepDn(char par, long *st);     // 

long Amp2Lev(long v);                //.    
long Lev2Amp(long v);                //.     

void Apply(char chan, char par);     //   
void ApplyK(int k, char s);          //  

//----------------------   : -------------------------

void Menu_Init(void)
{
  AttCode[OUT_A] = ATT_OFF;
  AttCode[OUT_B] = ATT_OFF;
  DacCode[OUT_A] = 0;
  DacCode[OUT_B] = 0;
  OutEnable = 0;      // 
  DispMenu = MNU_NO;  //   
  Menu = MNU_SPLASH;  //  splash screen
  Mnu_Splash(1);      //  splash screen
  SelChan = OUT_A;    // OUT A
  Lock = 0;           //  
  SvPre = 0;          //   
  RdPre = 0;          //   
  SvReq = SAVED;      //  
}

//----------------------------  : -------------------------------

void Menu_Exe(bool t)
{
  if(t)
  {
    if(MenuTimer) MenuTimer--;    //  
    if(ExitTimer) ExitTimer--;    //   
  }

  if(!Lock)                       //    , 
    KeyCode = Keyboard_GetCode(); //   , 
      else KeyCode = KEY_NO;      // KEY_NO

  if(KeyCode & REL)               //   , 
    KeyCode = KEY_NO;             // 

  bool MnuIni = Menu != DispMenu; //  
  switch(Menu)                    //  
  {
  case MNU_SPLASH: Mnu_Splash(MnuIni); break; //  
  case MNU_INFO:   Mnu_Info(MnuIni);   break; //   
  case MNU_OUTS:   Mnu_Outs(MnuIni);   break; // OUTS
  case MNU_STEPO:  Mnu_StepO(MnuIni);  break; //  OUTS
  case MNU_SERV1:  Mnu_Serv1(MnuIni);  break; // SERVICE, page 1
  case MNU_SERV2:  Mnu_Serv2(MnuIni);  break; // SERVICE, page 2
  case MNU_CALIB:  Mnu_Calib(MnuIni);  break; // CALIB
  case MNU_SHOW:   Mnu_Show(MnuIni);   break; // SHOW
  }

  if(KeyCode == (KEY_ESB | REP))  //   , 
  {
    Sound_High();                 //  
    SvReq = 0;                    //   0
    if(Menu == MNU_SERV1)
    {
      if(SubMenu == SERV1_SP)
        SvReq = SvPre;
      Menu = RetMenu;
    }
    KeyCode = KEY_NO;             // 
  }

  if(SvReq != SAVED && SvReq < PSS)
  {
    Vals_Save(SvReq);             // 
    SvReq = SAVED;
  }

  if(KeyCode != KEY_NO)           //   , 
  {
    Sound_Note();                 //  
    KeyCode = KEY_NO;             // 
  }
  Keyboard_SetCode(KeyCode);      //   
}

//----------------------------------------------------------------------------
//---------------------------  : -------------------------------
//----------------------------------------------------------------------------

//---------------------------- Splash screen: --------------------------------

void Mnu_Splash(bool ini)
{
  if(ini)                        //  , 
  {
    LCD_Picture(Splash);         // 
    LCD_Update();                // 
    Setup_Read();                // 
    Vals_Read(0);                //  0
    MenuTimer = ms2sys(SPL_TMV); //   splash screen
    DispMenu = Menu;             //  
  }
  if(KeyCode != KEY_NO ||        //   
     !MenuTimer)                 //  , 
  {
    Menu = MNU_INFO;             //  
  }
}

//-------------------------------- Info: -------------------------------------

void Mnu_Info(bool ini)
{
  if(ini)                        //  , 
  {
    Show_Info();                 // 
    Sound_Beep();                //beep
    MenuTimer = ms2sys(INF_TMV); //   splash screen
    DispMenu = Menu;             //  
  }
  if(KeyCode != KEY_NO ||        //   
     !MenuTimer)                 //  , 
  {
    OutEnable = 1;               // 
    Apply(OUT_A, PAR_A);         //  A
    Apply(OUT_B, PAR_A);         //  B
    Apply(OUT_A, PAR_R);         //  A
    Apply(OUT_B, PAR_R);         //  B
    Sound_Beep();                //beep
    Menu = MNU_OUTS;             //  OUTS
  }
}

//-------------------------   : ---------------------------

void Mnu_Outs(bool ini)
{
  static char __flash Mnu_StrA[] = {"OUT A"};
  static char __flash Mnu_StrB[] = {"OUT B"};
  static char __flash Mnu_StrM[] = {"Mode:\n\n\n\n"
                                    "Phase:"};
  static char __flash Str_L[2][7] = { {"Ampl.:"},
                                      {"V rms:"} };
  if(ini)                        //  , 
  {
    //  :
    LCD_Clear();                 // 
    LCD_Inv(1);                  // 
    LCD_PutString((SelChan == OUT_A)? Mnu_StrA : Mnu_StrB); // 
    LCD_Inv(0);                  // 
    LCD_SetLinPos(1, 8);         //    
    LCD_PutString(Mnu_StrM);     // 
    LCD_SetLinPos(6, 1);         //    
    LCD_PutString(&Str_L[Units[SelChan]][0]); //  
    // :
    SelPar = Pars[SelChan];      //   
    char n = SelChan;
    if(SelPar == PAR_M) n = OUT_A; //PAR_M    A
    if(Combo && SelPar == PAR_F) n = OUT_A; //PAR_   A,  Combo
    if(SelPar == PAR_A)
    {
      //  ,     ,
      //   :
      Buffer = Amp2Lev(ValOuts[n][PAR_A]);  // 
      MinVal = MIN_A;                       //min...
      MaxVal = Amp2Lev(MAX_A / ATT_0DB);    //max...
    }
    else
    {
      Buffer = ValOuts[n][SelPar];          //   
      MinVal = LimValOuts[SelPar][LIM_MIN]; //min...
      MaxVal = LimValOuts[SelPar][LIM_MAX]; //max...
    }
    Step = StpOuts[SelChan][SelPar];        //  
    // :
    Show_Val(PAR_M);             //
    Show_Val(PAR_S);             // 
    Show_Val(PAR_F);             //
    Show_Val(PAR_P);             //
    Show_Val(PAR_A);             //
    Show_Val(PAR_R);             //
    LCD_Update();                // 
    DispMenu = Menu;             //  
  }
  // "CHAN":
  if(KeyCode == KEY_CHAN)
  {
    if(SelChan == OUT_A)         //   A,
      SelChan = OUT_B;           //   B,
        else SelChan = OUT_A;    //   A
      DispMenu = MNU_NO;         //  
      Sound_Beep();              //beep
      KeyCode = KEY_NO;          // 
  }
  // "UP":
  if((KeyCode & ~REP) == KEY_UP)
  {
    char n = SelPar - 1;         // 
    if(SelPar == PAR_M) n = PAR_R;
    SelPar = n;
    Pars[SelChan] = n;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "DN":
  if((KeyCode & ~REP) == KEY_DN)
  {
    char n = SelPar + 1;         // 
    if(SelPar == PAR_R) n = PAR_M;
    SelPar = n;
    Pars[SelChan] = n;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "STEP":
  if(KeyCode == KEY_STEP)
  {
    if(LimStpOuts[SelPar][LIM_MAX] != 1) //   
    {
      Sound_High();              //high beep
      RetMenu = Menu;            //  
      SubMenu = SelPar;          //    
      Menu = MNU_STEPO;          //    
      KeyCode = KEY_NO;          // 
    }
  }
  // : 
  if(Edit_Val(KeyCode))          //.   
  {
    if(SelPar == PAR_M ||
      (Combo && SelPar == PAR_F))
      ValOuts[OUT_A][SelPar] = Buffer; //  PAR_M  PAR_F
    else if(SelPar == PAR_A)
      ValOuts[SelChan][SelPar] = Lev2Amp(Buffer); // 
    else ValOuts[SelChan][SelPar] = Buffer; // 
    Apply(SelChan, SelPar);      //   
    Show_Val(SelPar);            //   
    LCD_Update();
    if(SelPar == PAR_M ||        //   ,
       SelPar == PAR_S ||        //
       SelPar == PAR_R)          // ,          
      DispMenu = MNU_NO;         //  
    KeyCode = KEY_NO;            // 
  }
  // :
  if(KeyCode == KEY_ESB)
  {
    RetMenu = Menu;              //  
    SubMenu = SERV1_SM;
    Menu = MNU_SERV1;            // 
    Sound_Beep();                //beep
    KeyCode = KEY_NO;            // 
  }
}

//-------------------------   : -----------------------------

void Mnu_StepO(bool ini)
{
  static char __flash Mnu_StrA[] = {"OUT A"};
  static char __flash Mnu_StrB[] = {"OUT B"};
  static char __flash Mnu_StrS[] = {"Step:"};
  static char __flash Mnu_StrM[] = {"Freq.:\n"
                                    "Phase:\n"
                                    "Ampl.:"};
  static char __flash Mnu_StrR[] = {"Freq.:\n"
                                    "Phase:\n"
                                    "V rms:"};
  if(ini)                        //  , 
  {
    //  :
    LCD_Clear();                 // 
    LCD_Inv(1);                  // 
    LCD_PutString((SelChan == OUT_A)? Mnu_StrA : Mnu_StrB); // 
    LCD_SetLinPos(1, 8);         //   Step
    LCD_PutString(Mnu_StrS);     // Step
    LCD_Inv(0);                  // 
    LCD_SetLinPos(3, 1);         //    
    LCD_PutString((Units[SelChan] == UN_RMS)? Mnu_StrR : Mnu_StrM);
    // :
    Show_Step(PAR_F);
    Show_Step(PAR_P);
    Show_Step(PAR_A);
    LCD_Update();
    ExitTimer = ms2sys(EXI_TMV); //   
    DispMenu = Menu;             //  
  }
  // :
  if(KeyCode != KEY_NO)
  {
    ExitTimer = ms2sys(EXI_TMV); //   
  }
  // "UP":
  if((KeyCode & ~REP) == KEY_UP)
  {
    if(SubMenu == PAR_F)
      SubMenu = PAR_A;           
        else SubMenu--;          // 
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "DN":
  if((KeyCode & ~REP) == KEY_DN)
  {
    if(SubMenu == PAR_A)
      SubMenu = PAR_F;
        else SubMenu++;          // 
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "STEP":
  if(KeyCode == KEY_STEP)
  {
    Sound_Beep();                //beep
    Menu = RetMenu;              //   
    KeyCode = KEY_NO;            // 
  }
  //  
  if(Edit_Step(KeyCode))         //.  
  {
    StpOuts[SelChan][SubMenu] = Step; //  
    Show_Step(SubMenu);          //  
    LCD_Update();
    KeyCode = KEY_NO;            // 
  }
  // :
  if(KeyCode == KEY_ESB)
  {
    Sound_Beep();                //beep
    Menu = RetMenu;              //   
    KeyCode = KEY_NO;            // 
  }
  // :
  if(!ExitTimer)
  {
    Sound_Beep();                //beep
    Menu = RetMenu;              //   
  }
}

//------------------------  , page 1: -----------------------------

void Mnu_Serv1(bool ini)
{
  static char __flash Mnu_StrV[] = {"SERVICE 1"};
  static char __flash Mnu_StrM[] = {"Sound mode :\n"
                                    "Read preset:\n"
                                    "Save preset:\n"
                                    "Calibration:\n"  
                                    "Page 2     :"};
  if(ini)                        //  , 
  {
    //  :
    LCD_Clear();                 // 
    LCD_Inv(1);                  // 
    LCD_PutString(Mnu_StrV);     // SERVICE
    LCD_Inv(0);                  // 
    LCD_SetLinPos(2, 1);         //    
    LCD_PutString(Mnu_StrM);     //  
    // :
    Show_Serv1(SERV1_SM);        // 
    Show_Serv1(SERV1_RP);        //   
    Show_Serv1(SERV1_SP);        //   
    Show_Serv1(SERV1_CA);        //
    Show_Serv1(SERV1_P2);        //page 2
    LCD_Update();                // 
    DispMenu = Menu;             //  
  }
  // "CHAN":
  if(KeyCode == KEY_CHAN)
  {
    Menu = RetMenu;              //
    Sound_Beep();                //beep
    KeyCode = KEY_NO;            // 
  }
  // "UP":
  if((KeyCode & ~REP) == KEY_UP)
  {
    if(SubMenu > 0) SubMenu--;   // 
      else SubMenu = SERV1_MAX - 1;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "DN":
  if((KeyCode & ~REP) == KEY_DN)
  {
    if(SubMenu < SERV1_MAX - 1) SubMenu++; // 
      else SubMenu = 0;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  //   
  if(KeyCode == KEY_EUP)
  {
    bool upd = 0;
    switch(SubMenu)
    {
    case SERV1_SM:
      if(Sound_Mute()) { Sound_En(1); EMute = 0; upd = 1; }
      break;
    case SERV1_RP:
      if(RdPre < PSS - 1) { RdPre++; upd = 1; }
      break;
    case SERV1_SP:
      if(SvPre < PSS - 1) { SvPre++; upd = 1; }
      break;
    }
    if(upd)
    {
      Show_Serv1(SubMenu);       // 
      LCD_Update();              // 
      KeyCode = KEY_NO;          // 
    }
  }
  //   
  if(KeyCode == KEY_EDN)
  {
    bool upd = 0;
    switch(SubMenu)
    {
    case SERV1_SM:
      if(!Sound_Mute()) { Sound_En(0); EMute = 1; upd = 1; }
      break;
    case SERV1_RP:
      if(RdPre > 0) { RdPre--; upd = 1; }
      break;
    case SERV1_SP:
      if(SvPre > 0) { SvPre--; upd = 1; }
      break;
    }
    if(upd)
    {
      Show_Serv1(SubMenu);       // 
      LCD_Update();              // 
      KeyCode = KEY_NO;          // 
    }
  }
  // :
  if(KeyCode == KEY_ESB)
  {
    switch(SubMenu)
    {
    case SERV1_SM: Sound_Beep();        //beep
                   Menu = RetMenu;      // 
                   break;
    case SERV1_RP: if(Vals_Read(RdPre)) Sound_High(); //     
                     else Sound_Bell(); //  
                   Menu = RetMenu;      //
                   break;
    case SERV1_CA: Sound_Beep();        //beep
                   SubMenu = 0;         //
                   Menu = MNU_CALIB;    // 
                   break;
    case SERV1_P2: Sound_Beep();        //beep
                   SubMenu = 0;         //
                   Menu = MNU_SERV2;    //page 2
                   break;
    }
    KeyCode = KEY_NO;            // 
  }
  //  :
  if(KeyCode == (KEY_ESB | REP))
  {
    if(SubMenu == SERV1_SP)
    {
      Sound_High();              //  
      SvReq = SvPre;             //  
      Menu = RetMenu;            //
      KeyCode = KEY_NO;          // 
    }
  }
}

//------------------------  , page 2: -----------------------------

void Mnu_Serv2(bool ini)
{
  static char __flash Mnu_StrV[] = {"SERVICE 2"};
  static char __flash Mnu_StrM[] = {"Page 1     :\n"
                                    "Level A    :\n"
                                    "Level B    :\n"
                                    "Show info  :\n"
                                    "Exit       :"};
  if(ini)                        //  , 
  {
    //  :
    LCD_Clear();                 // 
    LCD_Inv(1);                  // 
    LCD_PutString(Mnu_StrV);     // SERVICE
    LCD_Inv(0);                  // 
    LCD_SetLinPos(2, 1);         //    
    LCD_PutString(Mnu_StrM);     //  
    // :
    Show_Serv2(SERV2_P1);        //page 1
    Show_Serv2(SERV2_LA);        //level A
    Show_Serv2(SERV2_LB);        //level B
    Show_Serv2(SERV2_SI);        //
    Show_Serv2(SERV2_EX);        //exit
    LCD_Update();                // 
    DispMenu = Menu;             //  
  }
  // "CHAN":
  if(KeyCode == KEY_CHAN)
  {
    Menu = RetMenu;              //
    Sound_Beep();                //beep
    KeyCode = KEY_NO;            // 
  }
  // "UP":
  if((KeyCode & ~REP) == KEY_UP)
  {
    if(SubMenu > 0) SubMenu--;   // 
      else SubMenu = SERV2_MAX - 1;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "DN":
  if((KeyCode & ~REP) == KEY_DN)
  {
    if(SubMenu < SERV2_MAX - 1) SubMenu++; // 
      else SubMenu = 0;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  //   
  if(KeyCode == KEY_EUP)
  {
    bool upd = 0;
    switch(SubMenu)
    {
    case SERV2_LA:
      if(Units[OUT_A] != UN_RMS)
      {
        Units[OUT_A] = UN_RMS;
        EUnits[OUT_A] = UN_RMS;
        upd = 1;
      }
      break;
    case SERV2_LB:
      if(Units[OUT_B] != UN_RMS)
      {
        Units[OUT_B] = UN_RMS;
        EUnits[OUT_B] = UN_RMS;
        upd = 1;
      }
      break;
    }
    if(upd)
    {
      Show_Serv2(SubMenu);       // 
      LCD_Update();              // 
      KeyCode = KEY_NO;          // 
    }
  }
  //   
  if(KeyCode == KEY_EDN)
  {
    bool upd = 0;
    switch(SubMenu)
    {
    case SERV2_LA:
      if(Units[OUT_A] == UN_RMS)
      {
        Units[OUT_A] = UN_AMP;
        EUnits[OUT_A] = UN_AMP;
        upd = 1;
      }
      break;
    case SERV2_LB:
      if(Units[OUT_B] == UN_RMS)
      {
        Units[OUT_B] = UN_AMP;
        EUnits[OUT_B] = UN_AMP;
        upd = 1;
      }
      break;
    }
    if(upd)
    {
      Show_Serv2(SubMenu);       // 
      LCD_Update();              // 
      KeyCode = KEY_NO;          // 
    }
  }
  // :
  if(KeyCode == KEY_ESB)
  {
    switch(SubMenu)
    {
    case SERV2_P1: Sound_Beep();        //beep
                   SubMenu = SERV1_P2;  //
                   Menu = MNU_SERV1;    //page 1
                   break;
    case SERV2_SI: Sound_Beep();        //beep
                   SubMenu = MNU_INFO;  //
                   Menu = MNU_SHOW;     // 
                   break;
    case SERV2_LA:
    case SERV2_LB:
    case SERV2_EX: Sound_Beep();        //beep
                   Menu = RetMenu;      // 
                   break;
    }
    KeyCode = KEY_NO;            // 
  }
}

//---------------------------  : -------------------------------

void Mnu_Calib(bool ini)
{
  static char __flash Mnu_StrS[] = {"CALIB"};
  static char __flash Mnu_StrM[] = {"Frequency:\n"
                                    "Level A  :\n"
                                    "Level B  :\n"
                                    "Save     :\n"
                                    "Exit     :"};
  if(ini)                        //  , 
  {
    //  :
    LCD_Clear();                 // 
    LCD_Inv(1);                  // 
    LCD_PutString(Mnu_StrS);     // CALIB
    LCD_Inv(0);                  // 
    LCD_SetLinPos(2, 1);         //    
    LCD_PutString(Mnu_StrM);     //  
    // :
    Show_Calib(CAL_F);           // 
    Show_Calib(CAL_A);           //  A
    Show_Calib(CAL_B);           //  B
    Show_Calib(CAL_S);           // 
    Show_Calib(CAL_E);           //  
    LCD_Update();                // 
    BuffK = Control_GetK(SubMenu); //  
    DispMenu = Menu;             //  
  }
  // "CHAN":
  if(KeyCode == KEY_CHAN)
  {
    Control_ReadCal();           //    EEPROM
    ApplyK(Control_GetK(CAL_F), CAL_F); //   A  B
    ApplyK(Control_GetK(CAL_A), CAL_A); //   A
    ApplyK(Control_GetK(CAL_B), CAL_B); //   B
    Menu = RetMenu;              //
    Sound_Beep();                //beep
    KeyCode = KEY_NO;            // 
  }
  // "UP":
  if((KeyCode & ~REP) == KEY_UP)
  {
    if(SubMenu > 0) SubMenu--;   // 
      else SubMenu = CAL_MAX - 1;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  // "DN":
  if((KeyCode & ~REP) == KEY_DN)
  {
    if(SubMenu < CAL_MAX - 1) SubMenu++; // 
      else SubMenu = 0;
    DispMenu = MNU_NO;           //  
    Sound_Tick();                //  
    KeyCode = KEY_NO;            // 
  }
  //   
  if(KeyCode == KEY_EUP)
  {
    if(BuffK < MAX_CALIB)
    {
      BuffK++;
      ApplyK(BuffK, SubMenu);
      Show_Calib(SubMenu);       // 
      LCD_Update();              // 
      KeyCode = KEY_NO;          // 
    }
  }
  //   
  if(KeyCode == KEY_EDN)
  {
    if(BuffK > MIN_CALIB)
    {
      BuffK--;
      ApplyK(BuffK, SubMenu);
      Show_Calib(SubMenu);       // 
      LCD_Update();              // 
      KeyCode = KEY_NO;          // 
    }
  }
  // :
  if(KeyCode == KEY_ESB)
  {
    if(SubMenu != CAL_S)
    {
      Control_ReadCal();          //    EEPROM
      ApplyK(Control_GetK(CAL_F), CAL_F); //   A  B
      ApplyK(Control_GetK(CAL_A), CAL_A); //   A
      ApplyK(Control_GetK(CAL_B), CAL_B); //   B
      Sound_Beep();              //beep
      Menu = RetMenu;            //
    }
    KeyCode = KEY_NO;            // 
  }
  //  :
  if(KeyCode == (KEY_ESB | REP))
  {
    if(SubMenu == CAL_S)
    {
      Sound_High();              //  
      Control_SaveCal();         //     
      Menu = RetMenu;            //
      KeyCode = KEY_NO;          // 
    }
  }
}

//------------------------------ Menu Show: ----------------------------------

void Mnu_Show(bool ini)
{
  if(ini)                        //  , 
  {
    if(SubMenu == MNU_INFO)
      Show_Info();               //  INFO
    if(SubMenu == MNU_SPLASH)
      LCD_Picture(Splash);       // SPLASH
    LCD_Update();                // 
    DispMenu = Menu;             //  
  }
  if(KeyCode != KEY_NO)          //  
  {
    if(SubMenu == MNU_INFO)
      SubMenu = MNU_SPLASH;      //  SPLASH
    else if(SubMenu == MNU_SPLASH)
      Menu = RetMenu;            //  
    DispMenu = MNU_NO;           //  
    Sound_Beep();                //beep
    KeyCode = KEY_NO;            // 
  }
}

//----------------------------------------------------------------------------
//----------------------------   EEPROM: ------------------------------
//----------------------------------------------------------------------------

//----------------------    EEPROM: --------------------------

void Setup_Read(void)
{
  if(ESigSet != SIGSET) //  ,
  {
    
    Contr = NOM_CONTR;  //   
    Setup_Save();       //   EEPROM
    EMute = 0;
    Sound_En(1);        /// 
    Units[OUT_A] = UN_AMP;
    Units[OUT_B] = UN_AMP;
    EUnits[OUT_A] = UN_AMP;
    EUnits[OUT_B] = UN_AMP;
  }
  else //  ,
  {
    Contr = EContr;     // 
    Sound_En(!EMute);   /// 
    Units[OUT_A] = EUnits[OUT_A];
    Units[OUT_B] = EUnits[OUT_B];
  }
  LCD_SetContr(Contr);  //  LCD
}

//--------------------    EEPROM: -------------------------

void Setup_Save(void)
{
  if(EContr != Contr) EContr = Contr;     //  LCD
  if(ESigSet != SIGSET) ESigSet = SIGSET; // 
}

//---------------------    EEPROM: -------------------------

bool Vals_Read(char n)
{
  bool valid = ESigPar[n] == SIGPAR;
  if(!valid) //  ,
  {
    if(n == 0)
    {
      //    :
      Pars[OUT_A] = PAR_F;
      Pars[OUT_B] = PAR_F;
      //  :
      SelChan = OUT_A;
      //  OUTS   :
      for(char i = 0; i < OUTS; i++)
        for(char j = 0; j < POUTS_MAX; j++)
        {
          StpOuts[i][j] = LimStpOuts[j][LIM_NOM];
          ValOuts[i][j] = LimValOuts[j][LIM_NOM]; // - 
        }
      SvReq = 0; //   0
      valid = 1;
    }
  }
  else //  ,
  {
    char d;
    //    :
    d = EPars[n][OUT_A]; if(d > POUTS_MAX - 1) d = POUTS_MAX - 1; Pars[OUT_A] = d;
    d = EPars[n][OUT_B]; if(d > POUTS_MAX - 1) d = POUTS_MAX - 1; Pars[OUT_B] = d;
    //    OUTS:
    for(char i = 0; i < OUTS; i++)
      for(char j = 0; j < POUTS_MAX; j++)
      {
        long v = EStpOuts[n][i][j];
        if(v < LimStpOuts[j][LIM_MIN]) v = LimStpOuts[j][LIM_MIN];
        if(v > LimStpOuts[j][LIM_MAX]) v = LimStpOuts[j][LIM_MAX];
        StpOuts[i][j] = v;
        v = EValOuts[n][i][j];
        if(v < LimValOuts[j][LIM_MIN]) v = LimValOuts[j][LIM_MIN];
        if(v > LimValOuts[j][LIM_MAX]) v = LimValOuts[j][LIM_MAX];
        ValOuts[i][j] = v;
      }
  }
  if(valid)
  {
    Apply(OUT_A, PAR_M); // PAR_M, PAR_F   A, B
    Apply(OUT_A, PAR_S); // PAR_S   A
    Apply(OUT_B, PAR_S); // PAR_S   A
    Apply(OUT_A, PAR_P); // PAR_P   A
    Apply(OUT_B, PAR_P); // PAR_P   B
    Apply(OUT_A, PAR_A); // PAR_A   A
    Apply(OUT_B, PAR_A); // PAR_A   B
    Apply(OUT_A, PAR_R); // PAR_R   A
    Apply(OUT_B, PAR_R); // PAR_R   B
  }
  return(valid);
}

//-------------------    EEPROM: ------------------------

void Vals_Save(char n)
{
  //  :
  if(EPars[n][OUT_A] != Pars[OUT_A]) EPars[n][OUT_A] = Pars[OUT_A];
  if(EPars[n][OUT_B] != Pars[OUT_B]) EPars[n][OUT_B] = Pars[OUT_B];
  //    outs:
  for(char i = 0; i < OUTS; i++)
    for(char j = 0; j < POUTS_MAX; j++)
    {
      if(EStpOuts[n][i][j] != StpOuts[i][j]) EStpOuts[n][i][j] = StpOuts[i][j];
      if(EValOuts[n][i][j] != ValOuts[i][j]) EValOuts[n][i][j] = ValOuts[i][j];
      __watchdog_reset();
    }
    __watchdog_reset();
  // :
  if(ESigPar[n] != SIGPAR) ESigPar[n] = SIGPAR;
}

//----------------------------------------------------------------------------
//-----------------------  : -------------------------
//----------------------------------------------------------------------------

//-------------------   : ---------------------

//key -  
//Buffer -  
//Step -   
//MinVal, MaxVal -    
//SelChan -  
//SelPar -  

bool Edit_Val(char key)
{
  //     :
  if(SelPar == PAR_A && (signed char)ValOuts[SelChan][PAR_R] == REL_OFF)
    return(0);
  if(key == KEY_EUP) // KEY_EUP
  {
    long v = Buffer;
    if(Buffer > 0)
      ValUp(SIGN_PLUS);
        else ValDn(SIGN_MINUS);
    //     :
    long max = MaxVal;
    if(SelPar == PAR_A)
    {
      if((signed char)ValOuts[SelChan][PAR_R] == REL_AUTO)
        max = max * ATT_0DB;
          else max = max * AttCode[SelChan]; 
    }
    if(Buffer > max) { Buffer = v; return(0); }
    return(1);
  }
  if(key == KEY_EDN) // KEY_EDN
  {
    long v = Buffer;
    v = Buffer;
    if(Buffer > 0)
      ValDn(SIGN_PLUS);
        else ValUp(SIGN_MINUS);
    if(Buffer < MinVal) { Buffer = v; return(0); }
    return(1);
  }
  return(0);
}

//-----------------------------  : -----------------------------------

//Buffer -  
//Step -   
//SelPar -  

void ValUp(bool sign)
{
  long step;
  if(Step) step = Step;
    else
    {
      long s = Buffer;
      if(SelPar == PAR_A) s = s / AttCode[SelChan];
      s = labs(s) / 100;
      step = 1;
      while(s)
      {
        s /= 10;
        StepUp(SelPar, &step);
      }
    }
  //    :  
  if(SelPar == PAR_A) step = step * AttCode[SelChan];  
  if(sign == SIGN_MINUS) step = -step;
  Buffer += step;
  long rem = Buffer % step;
  if(rem) Buffer -= rem;
}

//-----------------------------  : ------------------------------------

//Buffer -  
//Step -   
//SelPar -  

void ValDn(bool sign)
{
  long step;
  if(Step) step = Step;
    else
    {
      long s = Buffer;
      if(SelPar == PAR_A) s = s / AttCode[SelChan];
      s = (labs(s) - 1) / 100;
      step = 1;
      while(s)
      {
        s /= 10;
        StepUp(SelPar, &step);
      }
    }
  //    :  
  if(SelPar == PAR_A) step = step * AttCode[SelChan];  
  if(sign == SIGN_MINUS) step = -step;
  long rem = Buffer % step;
  if(rem) Buffer -= rem;
    else Buffer -= step;
}

//-------------------    : --------------------

long Amp2Lev(long v)
{
  if(Units[SelChan] != UN_AMP && (char)ValOuts[SelChan][PAR_S] == SH_S)
    v = (v * 14142L + 10000L) / 20000L;
  return(v);
}

//-------------------    : ---------------------

long Lev2Amp(long v)
{
  if(Units[SelChan] != UN_AMP && (char)ValOuts[SelChan][PAR_S] == SH_S)
    v = (v * 20000L + 7071L) / 14142L;
  return(v);
}

//----------------------------------------------------------------------------
//-------------------------  : -----------------------------
//----------------------------------------------------------------------------

//---------------------   : ------------------------

bool Edit_Step(char key)
{
  //     :
  if(SubMenu == PAR_A && (signed char)ValOuts[SelChan][PAR_R] == REL_OFF)
    return(0);
  if(key == KEY_EUP) return(StepUp(SubMenu, &Step));
  if(key == KEY_EDN) return(StepDn(SubMenu, &Step));
  return(0);
}

//---------------------------  : -------------------------------

bool StepUp(char par, long *st)
{
  if(*st == LimStpOuts[par][LIM_MAX]) return(0);
  if(*st == 0) *st = 1; else *st *= 10;
  return(1);
}

//---------------------------  : -------------------------------

bool StepDn(char par, long *st)
{
  if(*st == LimStpOuts[par][LIM_MIN]) return(0);
  if(*st == 1) *st = 0; else *st /= 10;
  return(1);
}

//----------------------------------------------------------------------------
//--------------------------   : ---------------------------
//----------------------------------------------------------------------------

//--------------------    : -------------------------

void Apply(char chan, char par)
{ 
  long v = ValOuts[chan][par];
  char r;
  long level;
  switch(par)
  {
  //  A.B  A+B:
  case PAR_M:
       Combo = ValOuts[OUT_A][PAR_M];
       if(Combo)
       {
         //   ,     
         //    :
         Control_SetF(OUTS, ValOuts[OUT_A][PAR_F]);  //A, B <- Fa
         Control_SetP(OUTS, 0);                      //A, B <- P = 0
         Control_SetP(OUT_A, ValOuts[OUT_A][PAR_P]); //A <- Pa
         Control_SetP(OUT_B, ValOuts[OUT_B][PAR_P]); //B <- Pb
       }
       else
       {
         //   ,    
         //  .   .
         Control_SetF(OUT_A, ValOuts[OUT_A][PAR_F]); //A <- Fa
         Control_SetF(OUT_B, ValOuts[OUT_B][PAR_F]); //B <- Fb
       }
       break;
  //   :     
  case PAR_S:
       Control_SetS(chan, v); //A(B) <- S
       Control_SetA(chan, DacCode[chan]); //A(B) <- A (recalibrate level)
       break;
  // :     
  case PAR_F:
       if(Combo) Control_SetF(OUTS, ValOuts[OUT_A][PAR_F]); //A, B <- F
         else Control_SetF(chan, v); //A(B) <- F
       break;
  // :     
  case PAR_P:
       Control_SetP(chan, v); //A(B) <- P
       break;
  // :     
  case PAR_A:
       r = AttCode[chan];
       if((signed char)ValOuts[chan][PAR_R] == REL_AUTO)
       {
         r = ATT_0DB;
         if(v < MAX_A / 10) r = ATT_20DB;
         if(v < MAX_A / 100) r = ATT_40DB;
         //     :
         if(OutEnable && AttCode[chan] != r)
         {
           Control_SetA(chan, 0); //A <- 0
           Control_SetR(chan, r); //A(B) <- R
         }
       }
       if(r != ATT_OFF)
       {
         AttCode[chan] = r; // AttCode
         AutoAtt[chan] = r; // AttCode  AUTO
         level = Amp2Lev(v);
         DacCode[chan] = Lev2Amp((level * 100 + r / 2) / r);
       }
       if(DacCode[chan] > MAX_DAC)
       {
         DacCode[chan] = MAX_DAC;  //  
         v = MAX_DAC / 100;        // 
         ValOuts[chan][PAR_A] = v; // 
       }
       if(OutEnable)
       {
         Control_SetA(chan, DacCode[chan]); //A(B) <- A
       }
       break;
  // :     
  case PAR_R:
       switch((signed char)ValOuts[chan][PAR_R])
       {
       case REL_AUTO: r = AutoAtt[chan]; break;                  
       case REL_OFF:  r = ATT_OFF;  break;                  
       case REL_40DB: r = ATT_40DB; break;                  
       case REL_20DB: r = ATT_20DB; break;
       default: r = ATT_0DB;
       }
       AttCode[chan] = r; // AttCode
       if(OutEnable)
       {
         level = Amp2Lev(DacCode[chan]);
         level = (level * r + 50) / 100;
         ValOuts[chan][PAR_A] = Lev2Amp(level);
         Control_SetA(chan, 0); //A <- 0
         Control_SetR(chan, r); //A(B) <- R
         Control_SetA(chan, DacCode[chan]); //A(B) <- A
       }
       break;
  }
}

//----------------   : -------------------

void ApplyK(int k, char s)
{
  Control_SetK(k, s); //  
  switch(s)
  {
  case CAL_F: Apply(OUT_A, PAR_F); //   A
              Apply(OUT_B, PAR_F); //   B
              break;
  case CAL_A: Apply(OUT_A, PAR_A); //   A
              break;
  case CAL_B: Apply(OUT_B, PAR_A); //   B
              break;
  }
}

//----------------------------------------------------------------------------
//----------------------------  : --------------------------------
//----------------------------------------------------------------------------

static char __flash Mnu_StrBL[] = {"..."};

//---------------------------   INFO: -------------------------------

void Show_Info(void)
{
  static char __flash Mnu_StrN[] = {"SINE WAVE GENERATOR"};
  static char __flash Mnu_StrT[] = {"SG-642"};
  static char __flash Mnu_StrV[] = {"Firmware version "VER};
  LCD_Clear();                        // 
  LCD_SetXY(10, 10);
  LCD_PutString(Mnu_StrN);            // 
  LCD_SetXY(37, 25);
  LCD_PutHString(Mnu_StrT);           // 
  LCD_SetXY(6, 45);
  LCD_PutString(Mnu_StrV);            // 
  LCD_Update();                       // 
}

//----------------------   : ---------------------------

void Show_Val(char par)
{
  static __flash char Str_M[2][4] = { {"A.B"},
                                      {"A+B"} };
  static __flash char Str_S[3][4] = { {"SIN"},
                                      {"SQR"} };
  static __flash char Str_R[5][6] = { {"Auto "},
                                      {" OFF "},
                                      {"-\24\20dB"},
                                      {"-\22\20dB"},
                                      {" \20 dB"} };
  LCD_Inv(par == SelPar);
  long v = ValOuts[SelChan][par];
  switch(par)
  {
  case PAR_M: LCD_SetLinPos(1, 14); v = ValOuts[OUT_A][PAR_M]; 
              LCD_PutString(&Str_M[(char)v][0]);
              break;
  case PAR_S: LCD_SetLinPos(1, 18); LCD_PutString(&Str_S[(char)v][0]);
              break;
  case PAR_F: LCD_SetXY(9, 20); if(Combo) v = ValOuts[OUT_A][PAR_F];
              Show_FreqH(v);
              break;
  case PAR_P: LCD_SetLinPos(5, 8); Show_Phase((int)v);
              break;
  case PAR_A: LCD_SetLinPos(6, 8); Show_Level(Amp2Lev(v));
              break;
  case PAR_R: LCD_SetLinPos(6, 16); char a = (char)v + 1;
              LCD_PutString(&Str_R[a][0]);
              break;
  }
  LCD_Inv(0);
}

//------------------------   : ------------------------------

void Show_Step(char par)
{
  LCD_Inv(par == SubMenu);
  long v = StpOuts[SelChan][par];
  switch(par)
  {
  case PAR_F: LCD_SetLinPos(3, 8); Show_FreqStep(v);
              break;
  case PAR_P: LCD_SetLinPos(4, 8); Show_PhaseStep((int)v);
              break;
  case PAR_A: LCD_SetLinPos(5, 8); Show_LevelStep(v * AttCode[SelChan]);
              break;
  }
  while(LCD_GetPos() < 21)
   LCD_PutChar(' ');
  LCD_Inv(0);
}

//------------------   , page 1: -----------------------

void Show_Serv1(char s)
{
  static char __flash Mnu_StrS0[] = {"OFF"};
  static char __flash Mnu_StrS1[] = {"ON "};
  LCD_Inv(s == SubMenu);
  switch(s)
  {
  case SERV1_SM: LCD_SetLinPos(2, 14);
                 LCD_PutString(Sound_Mute()? Mnu_StrS0 : Mnu_StrS1); 
                 break;
  case SERV1_RP: LCD_SetLinPos(3, 14); LCD_PutChar(RdPre + 0x10);
                 break;
  case SERV1_SP: LCD_SetLinPos(4, 14); LCD_PutChar(SvPre + 0x10);
                 break;
  case SERV1_CA: LCD_SetLinPos(5, 14); LCD_PutString(Mnu_StrBL);
                 break;
  case SERV1_P2: LCD_SetLinPos(6, 14); LCD_PutString(Mnu_StrBL);
                 break;
  }
  while(LCD_GetPos() < 21)
   LCD_PutChar(' ');
  LCD_Inv(0);
}

//------------------   , page 2: -----------------------

void Show_Serv2(char s)
{
  static char __flash Mnu_StrUA[] = {"Ampl."};
  static char __flash Mnu_StrUR[] = {"RMS  "};
  LCD_Inv(s == SubMenu);
  switch(s)
  {
  case SERV2_P1: LCD_SetLinPos(2, 14); LCD_PutString(Mnu_StrBL);
                 break;
  case SERV2_LA: LCD_SetLinPos(3, 14);
                 LCD_PutString((Units[OUT_A] == UN_RMS)? Mnu_StrUR : Mnu_StrUA); 
                 break;
  case SERV2_LB: LCD_SetLinPos(4, 14);
                 LCD_PutString((Units[OUT_B] == UN_RMS)? Mnu_StrUR : Mnu_StrUA); 
                 break;
  case SERV2_SI: LCD_SetLinPos(5, 14); LCD_PutString(Mnu_StrBL);
                break;
  case SERV2_EX: LCD_SetLinPos(6, 14); LCD_PutString(Mnu_StrBL);
                 break;
  }
  while(LCD_GetPos() < 21)
   LCD_PutChar(' ');
  LCD_Inv(0);
}

//--------------------   : --------------------------

void Show_Calib(char s)
{
  LCD_Inv(s == SubMenu);
  int k = Control_GetK(s);
  switch(s)
  {
  case CAL_F: LCD_SetLinPos(2, 12); Show_FCal(k); 
              break;
  case CAL_A: LCD_SetLinPos(3, 12); Show_ACal(k);
              break;
  case CAL_B: LCD_SetLinPos(4, 12); Show_ACal(k);
              break;
  case CAL_S: LCD_SetLinPos(5, 12); LCD_PutString(Mnu_StrBL);
              break;
  case CAL_E: LCD_SetLinPos(6, 12); LCD_PutString(Mnu_StrBL);
              break;
  }
  while(LCD_GetPos() < 21)
   LCD_PutChar(' ');
  LCD_Inv(0);
}

//---------------    0..50.000 000 kHz: ------------------

void Show_Freq(long f)
{
  Long2BCD(f, Buff);                      // 
  char ch = ' '; bool khz = 0;
  for(char i = 0; i < 8; i++)
  {
    char d = Buff[7 - i];
    if(ch != ' ' || d) ch = d + 0x10;      // 
    if(i == 4 && ch == ' ') ch = 0x10;     //    4
    if(ch != ' ') LCD_PutChar(ch);                      
    if(i == 4 && !khz) LCD_PutChar('.');  // ,  Hz
    if(i == 1 && ch != ' ')
      { LCD_PutChar('.'); khz = 1; }      // ,  kHz
   }
  LCD_PutChar(' ');
  if(khz) LCD_PutChar('k');
  LCD_PutChar('H');
  LCD_PutChar('z');
}

//--------    0..50.000 000 kHz ( ): ---------

void Show_FreqH(long f)
{
  Long2BCD(f, Buff);                      // 
  char ch = ' '; bool khz = 0;
  LCD_Inv(0);                             // 
  for(char i = 0; i < 8; i++)
  {
    char d = Buff[7 - i];
    if(ch != ' ' || d) ch = d + 0x30;     // 
    if(i == 4 && ch == ' ') ch = 0x30;    //    4
    LCD_PutHChar(ch);
    if(i == 4 && !khz) LCD_PutHChar('.'); // ,  Hz
    if(i == 1 && ch != ' ')
      { LCD_PutHChar('.'); khz = 1; }     // ,  kHz
  }
  LCD_SetXY(LCD_GetX() + 18, LCD_GetY() + 1);
  LCD_PutChar(' ');                       // 3- 
  LCD_SetXY(LCD_GetX() - 18, LCD_GetY());
  if(SelPar == PAR_F) LCD_Inv(1);         //  , 
  if(khz) LCD_PutChar('k');
  LCD_PutChar('H');
  LCD_PutChar('z');
  LCD_Inv(0);
}

//----------------------    : ------------------------

static char __flash Str_Auto[] = {"Auto"};

void Show_FreqStep(long t)
{
  if(t) Show_Freq(t);
    else LCD_PutString(Str_Auto);
}

//---------------------    360.0: -------------------------

void Show_Phase(int p)
{
  Int2BCD(p, Buff);                       // 
  bool ch = 0;
  if(p < 0) LCD_PutChar('-');
  if(Buff[3]) { LCD_PutChar(Buff[3] + BOLD); ch = 1; }
  if(ch || Buff[2]) LCD_PutChar(Buff[2] + BOLD);
  LCD_PutChar(Buff[1] + BOLD);
  LCD_PutChar('.');                      // 
  LCD_PutChar(Buff[0] + BOLD);
  LCD_PutChar('\6');                     // 
  while(LCD_GetPos() < 15)
    LCD_PutChar(' ');
}

//-----------------------    : --------------------------

void Show_PhaseStep(int v)
{
  if(v) Show_Phase(v);
    else LCD_PutString(Str_Auto);
}

//----------------------------  : ---------------------------------

void Show_Level(long v)
{
  Long2BCD(v, Buff);
  if(Buff[5])
  {
    LCD_PutChar(Buff[5] + BOLD);
    LCD_PutChar(Buff[4] + BOLD);
    LCD_PutChar('.');
    LCD_PutChar(Buff[3] + BOLD);
    LCD_PutChar(Buff[2] + BOLD);
    LCD_PutChar(' ');
    LCD_PutChar('V');
    return;
  }
  if(Buff[4])
  {
    LCD_PutChar(Buff[4] + BOLD);
    LCD_PutChar('.');
    LCD_PutChar(Buff[3] + BOLD);
    LCD_PutChar(Buff[2] + BOLD);
    LCD_PutChar(' ');
    LCD_PutChar('V');
    LCD_PutChar(' ');
    return;
  }
  if(Buff[3])
  {
    LCD_PutChar(Buff[3] + BOLD);
    LCD_PutChar(Buff[2] + BOLD);
    LCD_PutChar(Buff[1] + BOLD);
    LCD_PutChar(' ');
    LCD_PutChar('m');
    LCD_PutChar('V');
    LCD_PutChar(' ');
    return;
  }
  if(Buff[2])
  {
    LCD_PutChar(Buff[2] + BOLD);
    LCD_PutChar(Buff[1] + BOLD);
    LCD_PutChar('.');
    LCD_PutChar(Buff[0] + BOLD);
    LCD_PutChar(' ');
    LCD_PutChar('m');
    LCD_PutChar('V');
    return;
  }
  LCD_PutChar(Buff[1] + BOLD);
  LCD_PutChar('.');
  LCD_PutChar(Buff[0] + BOLD);
  LCD_PutChar(' ');
  LCD_PutChar('m');
  LCD_PutChar('V');
  LCD_PutChar(' ');
}

//---------------------    : ----------------------

void Show_LevelStep(long v)
{
  if(v || (signed char)ValOuts[SelChan][PAR_R] == REL_OFF)
    Show_Level(v);
      else LCD_PutString(Str_Auto);
}

//---------------    : -----------------

void Show_FCal(int v)
{
  Int2BCD(v, Buff);
  if(v < 0) LCD_PutChar('-');
  if(Buff[2]) LCD_PutChar(Buff[2] + BOLD);
  LCD_PutChar(Buff[1] + BOLD);
  LCD_PutChar('.');
  LCD_PutChar(Buff[0] + BOLD);
  LCD_PutChar(' ');
  LCD_PutChar('P');
  LCD_PutChar('P');
  LCD_PutChar('M');
}

//---------------    : ------------------

void Show_ACal(int v)
{
  Int2BCD(v, Buff);
  if(v < 0) LCD_PutChar('-');
  LCD_PutChar(Buff[2] + BOLD);
  LCD_PutChar('.');
  LCD_PutChar(Buff[1] + BOLD);
  LCD_PutChar(Buff[0] + BOLD);
  LCD_PutChar('%');
}

//----------------------  Long2BCD: ----------------------------

//    -:
//x -    (32 ,  )
//buff -   (10 )

void Long2BCD(unsigned long x, char *buff)
{
  for(char i = 0; i < 10; i++)
    buff[i] = 0;                      //  
  for(char i = 0; i < 32; i++)        //    
  {
    char c = (x >> 31) & 1;           // 
    x = x << 1;                       //  
    for(char p = 0; p < 10; p++)      //   
    {
      char s = buff[p];               // 
      s = (s << 1) | c; c = 0;        //   
      if(s > 9) { s += 0x06; c = 1; } // 
      s &= 0x0F;                      // 
      buff[p] = s;                    // 
    }
  }
}

//-----------------------  Int2BCD: ----------------------------

//    -:
//x -    (16 ,  )
//buff -   (5 )

void Int2BCD(int x, char *buff)
{
  for(char i = 0; i < 5; i++)
    buff[i] = 0;                      //  
  if(x < 0) x = -x;
  for(char i = 0; i < 16; i++)        //    
  {
    char c = (x >> 15) & 1;           // 
    x = x << 1;                       //  
    for(char p = 0; p < 5; p++)       //   
    {
      char s = buff[p];               // 
      s = (s << 1) | c; c = 0;        //   
      if(s > 9) { s += 0x06; c = 1; } // 
      s &= 0x0F;                      // 
      buff[p] = s;                    // 
    }
  }
}

//--------------------   : -----------------------

void Menu_SetLock(bool b)
{
  Lock = b;
}

//-----------------    : -------------------

bool Menu_GetLock(void)
{
  return(Lock);
}

//--------------------------  : ----------------------------

void Menu_SetPar(char ch, char par, long v)
{
  bool draw = par & DRAW;
  bool beep = par & BEEP;
  par &= ~(DRAW | BEEP);
  char menu = Menu;
  // OUTS:
  if(ch == OUT_A || ch == OUT_B)
  {
    if(par >= POUTS_MAX) return; //  
    if(v < LimValOuts[par][LIM_MIN]) return; // -   max
    if(v > LimValOuts[par][LIM_MAX]) return; // -   min
    ValOuts[ch][par] = v;        // 
    Apply(ch, par);              // 
    menu = MNU_OUTS;             // 
    if(draw)
    {
      SelChan = ch;              //  
      SelPar = par;              //  
      Pars[ch] = par;            //  
      DispMenu = MNU_NO;         //  
      Menu = menu;               //  
      Buffer = v;                //   
    }
  }
  // CALIB:
  if(ch == CALIB)
  { 
    switch(par)
    {
    case CAL_F: ApplyK((int)v, CAL_F); break; // 
    case CAL_A: ApplyK((int)v, CAL_A); break; //  A
    case CAL_B: ApplyK((int)v, CAL_B); break; //  B
    case CAL_S: Control_SaveCal(); break; //             
    }
    menu = MNU_CALIB;            // 
    if(draw)
    {
      RetMenu = MNU_OUTS;
      SubMenu = par;
      DispMenu = MNU_NO;         //  
      Menu = menu;               //  
    }
  }
  // SETUP:
  if(ch == SETUP)
  { 
    char p = (char)v;
    switch(par)
    {
    case SAVE_P:  SvReq = p; break;    // 
    case READ_P:  Vals_Read(p); break; // 
    case LCD_C:   Contr = p; LCD_SetContr(p); break; //
    case SAVE_S:  Setup_Save(); break; //             
    case UNITS_A: Units[OUT_A] = p; EUnits[OUT_A] = p;
                  break;            
    case UNITS_B: Units[OUT_B] = p; EUnits[OUT_B] = p;
                  break;            
    }
    DispMenu = MNU_NO;           //  
  }
  if(beep) Sound_Tick();         // ,   
}

//---------------------------  : ------------------------------

long Menu_GetPar(char ch, char par)
{
  // OUTS:
  if(ch == OUT_A || ch == OUT_B)
  {
    if(par >= POUTS_MAX) return(0); //  
    return(ValOuts[ch][par]);       // 
  }
  // CALIB:
  if(ch == CALIB)
  { 
    return(Control_GetK(par));
  }
  // SETUP:
  if(ch == SETUP)
  {
    if(par == LCD_C) return(Contr); // LCD
  }
  return(0);                        //  
}

//------------------    : ---------------------

char Menu_GetSelPar(void)
{
  return(SelPar);
}

//-------------------    : -----------------------

char Menu_GetSelChan(void)
{
  return(SelChan);
}

//----------------------------------------------------------------------------


