;----------------------------------------------------------------------------

;* Title	: SG-642 2-channel sine wave generator co-processor
;* Version	: 1.3
;* Target	: ATmega88

;----------------------------------------------------------------------------

.include "m88def.inc"
.include "macros.mac"
.include "Header.asm"

;------------------------- Interrupt Vectors: -------------------------------

.CSEG				;code segment
.org	0
	rjmp	Init		;reset vector

.org	OC1Aaddr

;Timer1 output compare A 
;interrupt (DDS implementation)
;To minimize execution time handle is located
;at vector address. It eliminates RJMP command.

.include "DDS2.asm"

;-------------------- Program Execution Starts Here: ------------------------

Init:	ldy	RAMEND	
	out	SPL,YL		;locate stack
	out	SPH,YH

        clr     zero            ;clear zero register

	rcall	iWdog		;start internal watchdog
	rcall	iPorts		;ports init
	rcall	iDDS		;DDS subsystem init
	rcall	iPort		;port init

	sei			;interrupts enable

;Main loop:

Main:	rcall   ePort           ;port polling
        rcall   eWdog           ;watchdog restart
	rjmp 	Main		;loop

;------------------------- Subroutines area: --------------------------------

;Watchdog init:

iWdog:	wdr
	ldi	temp,(1<<WDCE) | (1<<WDE)
	sts	WDTCSR,temp	
	ldi	temp,(1<<WDE) | (1<<WDP2)	
	sts	WDTCSR,temp	;watchdog enable, period 260 mS
	ret	

;----------------------------------------------------------------------------

;Watchdog restart:

eWdog:	wdr			;internal watchdog restart
	ret

;----------------------------------------------------------------------------

;Ports init:
	
iPorts:	ldi	temp,PUPB
	out	PORTB,temp	;init PORTB and on/off pullup
	ldi	temp,DIRB	
	out	DDRB,temp	;set PORTB direction
	
	ldi	temp,PUPC
	out	PORTC,temp	;init PORTC and on/off pullup
	ldi	temp,DIRC
	out	DDRC,temp	;set PORTC direction

	ldi	temp,PUPD
	out	PORTD,temp	;init PORTD and on/off pullup
	ldi	temp,DIRD
	out	DDRD,temp	;set PORTD direction
	ret

;----------------------------------------------------------------------------

.include "Port.asm"

;----------------------------------------------------------------------------
